<html lang="en">
<head>
  <title>Example</title>
</head>
<body>
<h1>Example</h1>
<p>Example page</p>
<img src="{{ url_for('static', filename='ili.gif') }}" alt="Ili Robot">
<img src="/static/ili.gif" alt="Ili Robot">
</body>
</html>
